@@ -4,7 +4,7 @@
    protocol... */
 
 /*
- * Copyright (c) 2004-2006,2009 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2017 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1999-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -25,18 +25,9 @@
  *   <info@isc.org>
  *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
-#ifndef lint
-static char copyright[] =
-"$Id: alloc.c,v 1.26.116.3 2009/09/01 20:32:28 dhankins Exp $ Copyright (c) 2004-2006,2009 Internet Systems Consortium, Inc.  All rights reserved.\n";
-#endif /* not lint */
+#include "dhcpd.h"
 
 #include <omapip/omapip_p.h>
 
@@ -59,29 +50,42 @@ int rc_history_count;
 static void print_rc_hist_entry (int);
 #endif
 
-VOIDPTR dmalloc (size, file, line)
-	unsigned size;
-	const char *file;
-	int line;
-{
+static int dmalloc_failures;
+static char out_of_memory[] = "Run out of memory.";
+
+void *
+dmalloc(size_t size, const char *file, int line) {
 	unsigned char *foo;
-	unsigned len;
-	int i;
-	VOIDPTR *bar;
+	size_t len;
+	void **bar;
 #if defined (DEBUG_MEMORY_LEAKAGE) || defined (DEBUG_MALLOC_POOL) || \
 		defined (DEBUG_MEMORY_LEAKAGE_ON_EXIT)
+	int i;
 	struct dmalloc_preamble *dp;
 #endif
 
 	len = size + DMDSIZE;
 	if (len < size)
-		return (VOIDPTR)0;
+		return NULL;
 
 	foo = malloc(len);
 
-	if (!foo)
-		return (VOIDPTR)0;
-	bar = (VOIDPTR)(foo + DMDOFFSET);
+	if (!foo) {
+		dmalloc_failures++;
+		if (dmalloc_failures > 10) {
+			/* In case log_fatal() returns here */
+			IGNORE_RET(write(STDERR_FILENO,
+					 out_of_memory,
+					 strlen(out_of_memory)));
+			IGNORE_RET(write(STDERR_FILENO, "\n", 1));
+			exit(1);
+		} else if (dmalloc_failures >= 10) {
+			/* Something went wrong beyond repair. */
+			log_fatal("Fatal error: out of memory.");
+		}
+		return NULL;
+	}
+	bar = (void *)(foo + DMDOFFSET);
 	memset (bar, 0, size);
 
 #if defined (DEBUG_MEMORY_LEAKAGE) || defined (DEBUG_MALLOC_POOL) || \
@@ -137,11 +141,8 @@ VOIDPTR dmalloc (size, file, line)
 	return bar;
 }
 
-void dfree (ptr, file, line)
-	VOIDPTR ptr;
-	const char *file;
-	int line;
-{
+void 
+dfree(void *ptr, const char *file, int line) {
 	if (!ptr) {
 		log_error ("dfree %s(%d): free on null pointer.", file, line);
 		return;
@@ -207,12 +208,8 @@ void dfree (ptr, file, line)
 /* For allocation functions that keep their own free lists, we want to
    account for the reuse of the memory. */
 
-void dmalloc_reuse (foo, file, line, justref)
-	VOIDPTR foo;
-	const char *file;
-	int line;
-	int justref;
-{
+void 
+dmalloc_reuse(void *foo, const char *file, int line, int justref) {
 	struct dmalloc_preamble *dp;
 
 	/* Get the pointer to the dmalloc header. */
@@ -270,6 +267,7 @@ void dmalloc_dump_outstanding ()
 	struct dmalloc_preamble *dp;
 #if defined(DEBUG_MALLOC_POOL)
 	unsigned char *foo;
+	int i;
 #endif
 
 	if (!dmalloc_cutoff_point)
@@ -322,7 +320,7 @@ void dmalloc_dump_outstanding ()
 				inhistory = 1;
 				if (!noted) {
 				    log_info ("  %s(%d): %ld", dp -> file,
-					      dp -> line, dp -> size);
+					      dp -> line, (long) dp -> size);
 				    noted = 1;
 				}
 				print_rc_hist_entry (i);
@@ -335,7 +333,8 @@ void dmalloc_dump_outstanding ()
 			if (!inhistory)
 #endif
 				log_info ("  %s(%d): %ld",
-					  dp -> file, dp -> line, dp -> size);
+					  dp -> file, dp -> line, 
+					  (long) dp -> size);
 		}
 #endif
 	}
@@ -539,7 +538,7 @@ isc_result_t omapi_object_allocate (omapi_object_t **o,
 		
 		/* Sanity check. */
 		if (tsize < sizeof (omapi_object_t))
-			return ISC_R_INVALIDARG;
+			return DHCP_R_INVALIDARG;
 		
 		foo = dmalloc (tsize, file, line);
 		if (!foo)
@@ -574,7 +573,7 @@ isc_result_t omapi_object_reference (omapi_object_t **r,
 				     const char *file, int line)
 {
 	if (!h || !r)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 
 	if (*r) {
 #if defined (POINTER_DEBUG)
@@ -582,7 +581,7 @@ isc_result_t omapi_object_reference (omapi_object_t **r,
 			   file, line);
 		abort ();
 #else
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	*r = h;
@@ -601,14 +600,14 @@ isc_result_t omapi_object_dereference (omapi_object_t **h,
 	omapi_object_t *p, *hp;
 
 	if (!h)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 
 	if (!*h) {
 #if defined (POINTER_DEBUG)
 		log_error ("%s(%d): dereference of null pointer!", file, line);
 		abort ();
 #else
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	
@@ -622,7 +621,7 @@ isc_result_t omapi_object_dereference (omapi_object_t **h,
 		abort ();
 #else
 		*h = 0;
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	
@@ -695,6 +694,13 @@ isc_result_t omapi_object_dereference (omapi_object_t **h,
 /*			if (!hp -> type -> freer) */
 				rc_register (file, line, h, hp,
 					     0, 1, hp -> type -> rc_flag);
+			if (handle_reference) {
+				if (omapi_handle_clear(hp->handle) != 
+				    ISC_R_SUCCESS) {
+					log_debug("Attempt to clear null "
+						  "handle pointer");
+				}
+			}
 			if (hp -> type -> destroy)
 				(*(hp -> type -> destroy)) (hp, file, line);
 			if (hp -> type -> freer)
@@ -740,7 +746,7 @@ isc_result_t omapi_buffer_reference (omapi_buffer_t **r,
 				     const char *file, int line)
 {
 	if (!h || !r)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 
 	if (*r) {
 #if defined (POINTER_DEBUG)
@@ -748,7 +754,7 @@ isc_result_t omapi_buffer_reference (omapi_buffer_t **r,
 			   file, line);
 		abort ();
 #else
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	*r = h;
@@ -761,14 +767,14 @@ isc_result_t omapi_buffer_dereference (omapi_buffer_t **h,
 				       const char *file, int line)
 {
 	if (!h)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 
 	if (!*h) {
 #if defined (POINTER_DEBUG)
 		log_error ("%s(%d): dereference of null pointer!", file, line);
 		abort ();
 #else
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	
@@ -782,7 +788,7 @@ isc_result_t omapi_buffer_dereference (omapi_buffer_t **h,
 		abort ();
 #else
 		*h = 0;
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 
@@ -820,7 +826,7 @@ isc_result_t omapi_typed_data_new (const char *file, int line,
 		len = OMAPI_TYPED_DATA_NOBUFFER_LEN + val;
 		if (len < val) {
 			va_end(l);
-			return ISC_R_INVALIDARG;
+			return DHCP_R_INVALIDARG;
 		}
 		break;
 	      case omapi_datatype_data:
@@ -828,7 +834,7 @@ isc_result_t omapi_typed_data_new (const char *file, int line,
 		len = OMAPI_TYPED_DATA_NOBUFFER_LEN + val;
 		if (len < val) {
 			va_end(l);
-			return ISC_R_INVALIDARG;
+			return DHCP_R_INVALIDARG;
 		}
 		break;
 	      case omapi_datatype_object:
@@ -837,7 +843,7 @@ isc_result_t omapi_typed_data_new (const char *file, int line,
 		break;
 	      default:
 		va_end (l);
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	}
 	va_end (l);
 
@@ -876,14 +882,14 @@ isc_result_t omapi_typed_data_reference (omapi_typed_data_t **r,
 					 const char *file, int line)
 {
 	if (!h || !r)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 
 	if (*r) {
 #if defined (POINTER_DEBUG)
 		log_error ("%s(%d): reference store into non-null pointer!", file, line);
 		abort ();
 #else
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	*r = h;
@@ -896,14 +902,14 @@ isc_result_t omapi_typed_data_dereference (omapi_typed_data_t **h,
 					   const char *file, int line)
 {
 	if (!h)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 
 	if (!*h) {
 #if defined (POINTER_DEBUG)
 		log_error ("%s(%d): dereference of null pointer!", file, line);
 		abort ();
 #else
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	
@@ -917,7 +923,7 @@ isc_result_t omapi_typed_data_dereference (omapi_typed_data_t **h,
 		abort ();
 #else
 		*h = 0;
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	
@@ -949,7 +955,7 @@ isc_result_t omapi_data_string_new (omapi_data_string_t **d, unsigned len,
 
 	nlen = OMAPI_DATA_STRING_EMPTY_SIZE + len;
 	if (nlen < len)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 	new = dmalloc (nlen, file, line);
 	if (!new)
 		return ISC_R_NOMEMORY;
@@ -963,14 +969,14 @@ isc_result_t omapi_data_string_reference (omapi_data_string_t **r,
 					  const char *file, int line)
 {
 	if (!h || !r)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 
 	if (*r) {
 #if defined (POINTER_DEBUG)
 		log_error ("%s(%d): reference store into non-null pointer!", file, line);
 		abort ();
 #else
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	*r = h;
@@ -983,14 +989,14 @@ isc_result_t omapi_data_string_dereference (omapi_data_string_t **h,
 					    const char *file, int line)
 {
 	if (!h)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 
 	if (!*h) {
 #if defined (POINTER_DEBUG)
 		log_error ("%s(%d): dereference of null pointer!", file, line);
 		abort ();
 #else
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	
@@ -1004,7 +1010,7 @@ isc_result_t omapi_data_string_dereference (omapi_data_string_t **h,
 		abort ();
 #else
 		*h = 0;
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 
@@ -1034,7 +1040,7 @@ isc_result_t omapi_value_reference (omapi_value_t **r,
 				    const char *file, int line)
 {
 	if (!h || !r)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 
 	if (*r) {
 #if defined (POINTER_DEBUG)
@@ -1042,7 +1048,7 @@ isc_result_t omapi_value_reference (omapi_value_t **r,
 			   file, line);
 		abort ();
 #else
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	*r = h;
@@ -1055,14 +1061,14 @@ isc_result_t omapi_value_dereference (omapi_value_t **h,
 				      const char *file, int line)
 {
 	if (!h)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 
 	if (!*h) {
 #if defined (POINTER_DEBUG)
 		log_error ("%s(%d): dereference of null pointer!", file, line);
 		abort ();
 #else
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	
@@ -1076,7 +1082,7 @@ isc_result_t omapi_value_dereference (omapi_value_t **h,
 		abort ();
 #else
 		*h = 0;
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	
@@ -1116,7 +1122,7 @@ isc_result_t omapi_addr_list_reference (omapi_addr_list_t **r,
 					  const char *file, int line)
 {
 	if (!h || !r)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 
 	if (*r) {
 #if defined (POINTER_DEBUG)
@@ -1124,7 +1130,7 @@ isc_result_t omapi_addr_list_reference (omapi_addr_list_t **r,
 			   file, line);
 		abort ();
 #else
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	*r = h;
@@ -1137,14 +1143,14 @@ isc_result_t omapi_addr_list_dereference (omapi_addr_list_t **h,
 					    const char *file, int line)
 {
 	if (!h)
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 
 	if (!*h) {
 #if defined (POINTER_DEBUG)
 		log_error ("%s(%d): dereference of null pointer!", file, line);
 		abort ();
 #else
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 	
@@ -1158,7 +1164,7 @@ isc_result_t omapi_addr_list_dereference (omapi_addr_list_t **h,
 		abort ();
 #else
 		*h = 0;
-		return ISC_R_INVALIDARG;
+		return DHCP_R_INVALIDARG;
 #endif
 	}
 
